@@ -24,26 +24,28 @@ class RawValueSourceSchema(BaseModel):
     """
 
     view: str
-    datetime_col: str
-    value_col: Optional[str] = None
-    location_col: Optional[str] = None
+    end_datetime_col_name: str
+    start_datetime_col_name: Optional[str] = None
+    location_col_name: Optional[str] = None
     where_clause: str
 
 
 def physical_visits(
+    timestamp_for_output: Literal["start", "end"] = "end",
     shak_code: Optional[int] = None,
     shak_sql_operator: Optional[str] = "=",
     where_clause: Optional[str] = None,
     where_separator: Optional[str] = "AND",
     n_rows: Optional[int] = None,
     return_value_as_visit_length_days: Optional[bool] = False,
-    visit_types: Optional[
-        list[Literal["admissions", "ambulatory_visits", "emergency_visits"]]
-    ] = None,
+    visit_types: list[
+        Literal["admissions", "ambulatory_visits", "emergency_visits"]
+    ] = ["admissions", "ambulatory_visits", "emergency_visits"],
 ) -> pd.DataFrame:
     """Load pshysical visits to both somatic and psychiatry.
 
     Args:
+        timestamp_for_output (Literal["start", "end"], optional): Whether to use the start or end timestamp for the output. Defaults to "end".
         shak_code (Optional[int], optional): SHAK code indicating where to keep/not keep visits from (e.g. 6600). Combines with
             shak_sql_operator, e.g. "!= 6600". Defaults to None, in which case all admissions are kept.
         shak_sql_operator (Optional[str], optional): Operator to use with shak_code. Defaults to "=".
@@ -51,7 +53,7 @@ def physical_visits(
         where_separator (Optional[str], optional): Separator between where-clauses. Defaults to "AND".
         n_rows (Optional[int], optional): Number of rows to return. Defaults to None.
         return_value_as_visit_length_days (Optional[bool], optional): Whether to return length of visit in days as the value for the loader. Defaults to False which results in value=1 for all visits.
-        visit_types (Optional[list[Literal["admissions", "ambulatory_visits", "emergency_visits"]]], optional): Whether to subset visits by visit types. Defaults to None.
+        visit_types (list[Literal["admissions", "ambulatory_visits", "emergency_visits"]]]): Which visit types to load. Defaults to ["admissions", "ambulatory_visits", "emergency_visits"].
 
     Returns:
         pd.DataFrame: Dataframe with all physical visits to psychiatry. Has columns dw_ek_borger and timestamp.
@@ -59,76 +61,81 @@ def physical_visits(
 
     # SHAK = 6600 ≈ in psychiatry
 
-    d = {
+    source_schemas = {
         "LPR3": RawValueSourceSchema(
             view="[FOR_LPR3kontakter_psyk_somatik_inkl_2021_feb2022]",
-            datetime_col="datotid_lpr3kontaktslut",
-            value_col="datotid_lpr3kontaktstart",
-            location_col="shakkode_lpr3kontaktansvarlig",
+            start_datetime_col_name="datotid_lpr3kontaktstart",
+            end_datetime_col_name="datotid_lpr3kontaktslut",
+            location_col_name="shakkode_lpr3kontaktansvarlig",
             where_clause="AND [Kontakttype] = 'Fysisk fremmøde'",
         ),
         "ambulatory_visits": RawValueSourceSchema(
             view="[FOR_besoeg_psyk_somatik_LPR2_inkl_2021_feb2022]",
-            datetime_col="datotid_slut",
-            value_col="datotid_start",
-            location_col="shakafskode",
+            start_datetime_col_name="datotid_start",
+            end_datetime_col_name="datotid_slut",
+            location_col_name="shakafskode",
             where_clause="AND ambbesoeg = 1",
         ),
         "emergency_visits": RawValueSourceSchema(
             view="[FOR_akutambulantekontakter_psyk_somatik_LPR2_inkl_2021_feb2022]",
-            datetime_col="datotid_slut",
-            value_col="datotid_start",
-            location_col="afsnit_stam",
+            start_datetime_col_name="datotid_start",
+            end_datetime_col_name="datotid_slut",
+            location_col_name="afsnit_stam",
             where_clause="",
         ),
         "admissions": RawValueSourceSchema(
             view="[FOR_indlaeggelser_psyk_somatik_LPR2_inkl_2021_feb2022]",
-            datetime_col="datotid_udskrivning",
-            value_col="datotid_indlaeggelse",
-            location_col="shakKode_kontaktansvarlig",
+            start_datetime_col_name="datotid_indlaeggelse",
+            end_datetime_col_name="datotid_udskrivning",
+            location_col_name="shakKode_kontaktansvarlig",
             where_clause="",
         ),
     }
 
-    if visit_types:
-        allowed_visit_types = ["admissions", "ambulatory_visits", "emergency_visits"]
-        if any(types not in allowed_visit_types for types in visit_types):
-            raise ValueError(
-                f"Invalid visit type. Allowed types of visits are {allowed_visit_types}.",
-            )
+    allowed_visit_types = ["admissions", "ambulatory_visits", "emergency_visits"]
 
-        english_to_lpr3_visit_type = {  # pylint: disable=invalid-name
-            "admissions": "'Indlæggelse'",
-            "ambulatory_visits": "'Ambulant'",
-            "emergency_visits": "'Akut ambulant'",
-        }
-        d = {key: d[key] for key in visit_types + ["LPR3"]}
-        english_to_lpr3_visit_type = [
-            english_to_lpr3_visit_type[visit] for visit in visit_types
-        ]
-        d[
-            "LPR3"
-        ].where_clause += f" AND pt_type IN ({','.join(english_to_lpr3_visit_type)})"
+    if any(types not in allowed_visit_types for types in visit_types):
+        raise ValueError(
+            f"Invalid visit type. Allowed types of visits are {allowed_visit_types}.",
+        )
+
+    english_to_lpr3_visit_type = {  # pylint: disable=invalid-name
+        "admissions": "'Indlæggelse'",
+        "ambulatory_visits": "'Ambulant'",
+        "emergency_visits": "'Akut ambulant'",
+    }
+    chosen_schemas = {
+        visit_type: source_schemas[visit_type] for visit_type in visit_types + ["LPR3"]
+    }
+    english_to_lpr3_visit_type = [
+        english_to_lpr3_visit_type[visit] for visit in visit_types
+    ]
+    chosen_schemas[
+        "LPR3"
+    ].where_clause += f" AND pt_type IN ({','.join(english_to_lpr3_visit_type)})"
 
     dfs = []
 
-    for schema in d.values():
-        cols = f"{schema.datetime_col}, dw_ek_borger"
+    for schema in chosen_schemas.values():
+        cols = f"{schema.start_datetime_col_name}, {schema.end_datetime_col_name}, dw_ek_borger"
 
-        if return_value_as_visit_length_days:
-            cols += f", {schema.value_col} AS value_col"
-
-        sql = f"SELECT {cols} FROM [fct].{schema.view} WHERE {schema.datetime_col} IS NOT NULL {schema.where_clause}"
+        sql = f"SELECT {cols} FROM [fct].{schema.view} WHERE {schema.start_datetime_col_name} IS NOT NULL {schema.where_clause}"
 
         if shak_code is not None:
-            sql += f" AND {schema.location_col} != 'Ukendt'"
-            sql += f" AND left({schema.location_col}, {len(str(shak_code))}) {shak_sql_operator} {str(shak_code)}"
+            sql += f" AND {schema.location_col_name} != 'Ukendt'"
+            sql += f" AND left({schema.location_col_name}, {len(str(shak_code))}) {shak_sql_operator} {str(shak_code)}"
 
         if where_clause is not None:
             sql += f" {where_separator} {where_clause}"
 
         df = sql_load(sql, database="USR_PS_FORSK", chunksize=None, n_rows=n_rows)
-        df.rename(columns={schema.datetime_col: "timestamp"}, inplace=True)
+        df.rename(
+            columns={
+                schema.end_datetime_col_name: "timestamp_end",
+                schema.start_datetime_col_name: "timestamp_start",
+            },
+            inplace=True,
+        )
 
         dfs.append(df)
 
@@ -137,26 +144,34 @@ def physical_visits(
 
     # 0,8% of visits are duplicates. Unsure if overlap between sources or errors in source data. Removing.
     output_df = output_df.drop_duplicates(
-        subset=["timestamp", "dw_ek_borger"],
+        subset=["timestamp_start", "timestamp_end", "dw_ek_borger"],
         keep="first",
     )
 
     # Change value column to length of admission in days
     if return_value_as_visit_length_days:
         output_df["value"] = (
-            output_df["timestamp"] - pd.to_datetime(output_df["value_col"])
+            output_df["timestamp_end"] - pd.to_datetime(output_df["timestamp_start"])
         ).dt.total_seconds() / 86400
-        output_df = output_df.drop(columns="value_col")
     else:
         output_df["value"] = 1
 
     log.info("Loaded physical visits")
 
-    return output_df.reset_index(drop=True)
+    output_df.rename(
+        columns={f"timestamp_{timestamp_for_output}": "timestamp"}, inplace=True
+    )
+
+    # Keep only one visit per timestamp
+    output_df = output_df.drop_duplicates(
+        subset=["dw_ek_borger", "timestamp"], keep="first"
+    )
+
+    return output_df[["dw_ek_borger", f"timestamp", "value"]].reset_index(drop=True)
 
 
-@data_loaders.register("visits")
-def visits(
+@data_loaders.register("physical_visits")
+def physical_visits_loader(
     n_rows: Optional[int] = None,
     return_value_as_visit_length_days: Optional[bool] = False,
 ) -> pd.DataFrame:
@@ -167,11 +182,12 @@ def visits(
     )
 
 
-@data_loaders.register("visits_psych")
-def visits_psych(
+@data_loaders.register("physical_visits_to_psychiatry")
+def physical_visits_to_psychiatry(
     n_rows: Optional[int] = None,
-    timestamps_only: bool = True,
-    return_value_as_visit_length_days: Optional[bool] = False,
+    timestamps_only: bool = False,
+    return_value_as_visit_length_days: Optional[bool] = True,
+    timestamp_for_output: Literal["start", "end"] = "start",
 ) -> pd.DataFrame:
     """Load physical visits to psychiatry."""
     df = physical_visits(
@@ -179,16 +195,17 @@ def visits_psych(
         shak_sql_operator="=",
         n_rows=n_rows,
         return_value_as_visit_length_days=return_value_as_visit_length_days,
+        timestamp_for_output=timestamp_for_output,
     )
 
     if timestamps_only:
-        df = df.drop("value", axis=1)
+        df.drop(columns=["value"], inplace=True)
 
     return df
 
 
-@data_loaders.register("visits_somatic")
-def visits_somatic(
+@data_loaders.register("physical_visits_to_somatic")
+def physical_visits_to_somatic(
     n_rows: Optional[int] = None,
     return_value_as_visit_length_days: Optional[bool] = False,
 ) -> pd.DataFrame:
@@ -252,14 +269,14 @@ def emergency_visits(
     )
 
 
-@data_loaders.register("amb_emerg_visits")
-def amb_emerg_visits(
+@data_loaders.register("ambulatory_and_emergency_visits")
+def ambulatory_and_emergency_visits(
     n_rows: Optional[int] = None,
     return_value_as_visit_length_days: Optional[bool] = False,
     shak_code: Optional[int] = None,
     shak_sql_operator: Optional[str] = None,
 ) -> pd.DataFrame:
-    """Load emergency and ambulatory visits."""
+    """Load emergency visits."""
     return physical_visits(
         visit_types=["ambulatory_visits", "emergency_visits"],
         return_value_as_visit_length_days=return_value_as_visit_length_days,
